dojo.provide("com.ibm.pb.debug.Tracer");

dojo.require("dojox.lang.aspect");

(function(){
	var argsExtractor = function(prefix, args) {
		var msg = (prefix ? prefix + " " : "") + (args[0] ? args[0] : "");
		return [msg].concat([args[1]]).concat([args[2]]).concat(args[3]);
	};
	
	var asp = dojox.lang.aspect;
	
	if(!console.log) {
		console.log = function(){};
	}
	if(!console.info) {
		console.info = console.log;
	}
	if(!console.warn) {
		console.warn = console.log;
	}
	if(!console.error) {
		console.error = console.log;
	}
	
	var TraceAdvice = function(context) {
		this.name = context.joinPoint.targetName;
		this.instance = context.instance;
		this.prefix = dojo.string.pad("", context.depth * 2, "--", true);
	};
	
	dojo.extend(TraceAdvice, {
		/* Logging helpers */
		log: function() {
			console.log.apply(console, argsExtractor(this.prefix, arguments)); 
		},
		info: function() {
			console.info.apply(console, argsExtractor(this.prefix, arguments));
		},
		warn: function() {
			console.warn.apply(console, argsExtractor(this.prefix, arguments));
		},
		error: function() {
			console.error.apply(console, argsExtractor(this.prefix, arguments));
		},
		
		/* Advice providers */
		/*
		before: function() {
			this.log("Before -> " + this.name + "(%o)", arguments);
		},
		*/
		around: function() {
			this.log("Before -> %o.%o(%o)", this.instance, this.name, arguments);
			var ret = asp.proceed.apply(null, arguments);
			this.log("After -> %o.%o -> Returned %o", this.instance, this.name, [ret]);
			return ret;
		},
		afterThrowing: function(err) {
			this.error("Exception in -> %o.%o: %o", this.instance, this.name, [err]);
		}
	});
	
	com.ibm.pb.debug.Tracer = {
		advise: asp.advise,
		unadvise: asp.unadvise,
		trace: function(obj, methods) {
			if(!methods) {methods = /^\S/;}
			return asp.advise(obj, methods, TraceAdvice);
		},
		untrace: function(handle) {
			asp.unadvise(handle);
		}
	};
})();
