dojo.provide("com.ibm.pb.tree.Traversal");

dojo.require("com.ibm.pb.logging.Loggable");

dojo.declare("com.ibm.pb.tree.Traversal", com.ibm.pb.logging.Loggable, {
	model: null,
	constructor: function(model) {
		this.model = model;
		this.stopped = true;
	},
	isStopped: function() {
		return this.stopped;
	},
	/**
	 * Finds the first node in the tree model that meets the test criteria
	 * and calls the callback with that single node.
	 */
	find: function(node, test, callback) {
		var mname = "find", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		this.collect(node, test, function(list){
			callback(list[0]);
		}, function(params){
			return params.list.length > 0;
		});
		lgr.exiting(mname, arguments);
	},
	/**
	 * Finds all the nodes in the tree model that meet the test criteria
	 * and calls the callback with an array of those nodes.  Supports
	 * conditionally stopping at any point using a stop test function.
	 */
	collect: function(node, test, callback, stop, params) {
		var mname = "collect", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		this.stopped = false;
		if(!params) {
			params = {};
		}
		if(!params.list) {
			params.list = [];
		}
		params.level = 1;
		this.visit(node, test, callback, stop || function(){return false;}, params);
		lgr.exiting(mname, arguments);
	},
	/**
	 * Visits a particular node, performing the test condition on the
	 * node and conditionally adding it to the list of found nodes that
	 * meet the test criteria.
	 */
	visit: function(node, test, callback, stop, params) {
		var mname = "visit", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		if(test(this.model, node)) {
			lgr.trace(mname, "Found positive testing node ${0}", [node]);
			params.list.push(node);
		}
		if(stop(params)) {
			this.stopped = true;
			lgr.trace(mname, "Hard stopping traversal and calling back with results ${0}", [params.list]);
			callback(params.list);
		}
		else {
			this.next(node, test, callback, stop, params);
		}
		lgr.exiting(mname, arguments);
	},
	/**
	 * Called after a node is visited in order to find and visit the next
	 * node in the traversal sequence so long as the traversal hasn't
	 * stopped.  Unimplemented and requires extensions using a particular
	 * traversal algorithm to implement.
	 */
	next: function(node, test, callback, stop, params) {
		
	}
});